#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import Queue
import threading
import logging
from datetime import datetime

from whoosh import index
from whoosh import writing
from whoosh.fields import Schema, STORED, NGRAM, TEXT, DATETIME, ID
from whoosh.qparser import QueryParser
from whoosh.analysis import NgramAnalyzer, NgramWordAnalyzer
from whoosh import highlight


schema = Schema(
    title=TEXT(stored=True),
    path=ID(stored=True, unique=True),  # TODO: non-alphabetical filename.
    content=NGRAM(stored=True, phrase=False, minsize=2, maxsize=4),
    date=DATETIME(stored=True)
    )

num_worker_threads = 1  # no need to use multiple threads
occured_error = None
# TODO: implements the way to reset error.


def register_worker():
    def commit(posted):
        try:
            ixpath = os.path.join(posted['indexroot'],
                                  posted['_index'])
            ix = open_index(ixpath)
            writer = ix.writer()

            writer.update_document(
                title=posted['title'],
                path=posted['path'],
                date=datetime.now(),
                content=posted['message']
                )
            writer.commit()
            ix.close()
        except index.IndexError, e:  # use for returning error
            print "IndexError", e
            occured_error = e
        except writing.IndexingError, e:
            print "IndexingError", e
            occured_error = e

    while True:
        posted = taskq.get()
        commit(posted)
        taskq.task_done()
        logging.info("commit done: left = {0}".format(taskq.qsize()))


def open_index(indexdir):
    if not os.path.exists(indexdir):
        os.makedirs(indexdir)
        index.create_in(indexdir, schema)

    return index.open_dir(indexdir)


def isErrorOccured():
    ''' check if error is occured or not'''
    return occured_error


def register(posted, indexroot):
    """ register to index about the posted document
    Note about register itself will be done by register worker.
    This function is return ok if no error is occured.
    """
    posted['indexroot'] = indexroot
    taskq.put(posted)
    logging.info("taskq put: left = {0}".format(taskq.qsize()))
    
    ret = {
        "ok": True,
        "_index": posted["_index"],
        "_path": posted["path"]
        }
    if isErrorOccured():
        ret['ok'] = False
        ret['message'] = str(occured_error)

    return ret


def search(_index, query, indexroot):
    if query is None:
        return {}
    ixpath = os.path.join(indexroot, _index)
    ix = open_index(ixpath)
    parser = QueryParser("content", schema=ix.schema)
    q = parser.parse(query.decode('utf-8'))

    ret = []
    with ix.searcher() as searcher:
        for r in searcher.search(q):
            # TODO: Without N-gramed text formatter
            # t = r.highlights('content', top=1)
            t = r['content']
            result = {
                "_index": _index,
                "_id": r.docnum,
                "_source": {
                "title": r['title'],
                "path": r['path'],
                "postDate": r['date'].isoformat(),
                "message": t
                }
            }
            ret.append(result)

    return ret

#
# Creating Task Queue due to commit is too slow
#

taskq = Queue.Queue()
for i in range(num_worker_threads):
    t = threading.Thread(target=register_worker)
    t.daemon = True
    t.start()


if __name__ == '__main__':
    ix = open_index("../index/test_rst")
    query = "テスト"
    parser = QueryParser("content", schema=ix.schema)
    q = parser.parse(query.decode('utf-8'))

    with ix.searcher() as searcher:
        for hit in searcher.search(q, terms=True):
            print "-----"
            print("Contains:", hit.matched_terms())
            print hit.highlights('content')





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import simplejson


class Config():
    port = 9876
    indexroot = None
    password = None
    host = "localhost"

    def __init__(self, conffile):
        self.read(conffile)
    
    def read(self, conffile):
        with open(conffile) as fp:
            d = simplejson.load(fp)
            self.port = d['port']
            self.indexroot = d['indexroot']
            self.host = d['host']
    
    
if __name__ == '__main__':
    pass




import os
import sys

from setuptools import setup, find_packages


requirements = [
    "bottle",
    "sphinx",
    "whoosh",
    "docutils",
    "simplejson",
    ]

classifiers = [
    "Development Status :: 3 - Alpha",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python :: 2.7",
    "Topic :: Database :: Database Engines/Servers",
    "Topic :: Software Development :: Documentation",
    "Topic :: Documentation"
    ]

setup(
    name='dipus',
    version="0.0.3",
    description='Dipus is a simple full-text search server using Whoosh for Sphinx',
    long_description=open("README.rst").read(),
    classifiers=classifiers,
    keywords=['sphinx', 'whoosh', 'text search'],
    author='WAKAYAMA shirou',
    author_email='shirou.faw at gmail.com',
    url='http://bitbucket.org/r_rudi/dipus',
    license='BSD License',
    install_requires=requirements,
    package_dir={'': 'src'},
    packages = [
    'dipus',
    ],
    )




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
import logging

from bottle import Bottle, route, run, request, response, abort
from bottle import SimpleTemplate
import simplejson

from docstore import register, search

import config


app = Bottle()
conf = None


def auth(password):
    ''' not implemented yet '''
    return True


@app.route('/')
def index():
    return "index"


@app.route('/<_index>', method='POST')
def updateDocument(_index):
    password = request.forms.get('password')

    if auth(password) is False:
        abort("Forbidden")

    path = request.forms.get('path')
    if not path:
        abort(400, "path is not set")

    message = request.forms.get('message')
    if not message:
        abort(400, "message is not set")

    title = request.forms.get('title')
    if not title:
        title = ""
        
    posted = {
        '_index': _index.decode('utf-8'),
        'path': path.decode('utf-8'),
        'message': message.decode('utf-8'),
        'title': title.decode('utf-8')
        }

    ret = register(posted, conf.indexroot)

    return simplejson.dumps(ret)


@app.route('/<_index>/_search')
def query(_index):
    password = request.forms.get('password')
    if auth(password) is False:
        abort("Forbidden")

    query = request.query.get('q')

    results = search(_index, query, conf.indexroot)

    ret = {
        "total": len(results),
        "hits": results
        }


    json_response = simplejson.dumps(ret)
    response.content_type = 'application/json; charset=utf-8'

    callback_function = request.GET.get('callback')
    if callback_function:
        json_response = ''.join([callback_function, '(', json_response, ')'])

    return json_response


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Dipus: simple full-text search server')
    parser.add_argument('-c', '--config', nargs=1,
                        dest='conffile', action='store',
                        help='Config file path')
    args = parser.parse_args()

    if args.conffile is None:  # FIXME: why nargs ignored?
        parser.print_help()
        exit(0)

    conf = config.Config(args.conffile[0])
    
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)s %(message)s')

    run(app,
        host=conf.host, port=conf.port,
        reloader=True)
